function u = implTrapezoidalRule(params, u0, t, Matrices)
 
writeFrequency = params.write.frequency ;
Nxi = params.geometry.Nxi       ;
nTimeSteps = length(t) ;
nSavedTimeSteps = ceil(nTimeSteps/writeFrequency) ;

switch params.projection.mode
    case 'PODROM'
        S = load(params.projection.modeFile) ;
        U = S.U ;
        r = size(U,2) ;
        clear S
        [A, B] = Matrices(u0) ;
        Er = eye(r) ;
        Ar = U'*A*U ;
        Br = U'*B ;
        u = NaN(r, nSavedTimeSteps) ;
        u(:,1) = U'*u0 ;
        uTempOld = u(:,1) ;
        g = params.data.BC.g ;
        for i=2:nTimeSteps
            dt = t(i)-t(i-1) ;
            uTempNew = (Er-0.5*dt*Ar)\((Er+0.5*dt*Ar)*uTempOld+0.5*dt*Br*(g(t(i-1))+g(t(i)))) ;
            if(~mod(i-1,writeFrequency))
                currentIndex = (i-1)/writeFrequency ;
                u(:,1+currentIndex) = uTempNew ;
            end
            uTempOld = uTempNew ;
        end
        u = U*u ;
    case 'off'
        u = NaN(Nxi-2, nSavedTimeSteps) ;
        u(:,1) = u0 ;
        [A, B] = Matrices(u0) ;
        uTempOld = u(:,1) ;
        E = speye(Nxi-2) ;
        g = params.data.BC.g ;
        for i=2:nTimeSteps
            dt = t(i)-t(i-1) ;
            uTempNew = (E-0.5*dt*A)\((E+0.5*dt*A)*uTempOld+0.5*dt*B*(g(t(i-1))+g(t(i)))) ;
            if(~mod(i-1,writeFrequency))
                currentIndex = (i-1)/writeFrequency ;
                u(:,1+currentIndex) = uTempNew ;
            end
            uTempOld = uTempNew ;
        end
    case 'sPODROM'
        S = load(params.projection.modeFile) ;
        U = S.U ;
        shiftMatrixGenerator = S.shiftMatrixGenerator ;
        NxiExtended = size(U, 1) ;
        neededPointsOnTheLeft = S.neededPointsOnTheLeft ;
        r = size(U, 2) ;
        clear S
        [A, B] = Matrices(u0) ;
        u = NaN(Nxi-2, nSavedTimeSteps) ;
        u(:,1) = u0 ;
        ur = zeros(r+1, nSavedTimeSteps) ;
        UNxi = U(neededPointsOnTheLeft+(1:Nxi-2),:) ;
        ur(1:end-1,1) = (UNxi'*UNxi)\UNxi'*u0 ;
        urTempOld = ur(:,1) ;
        g = params.data.BC.g ;
        params.geometry.Nxi = params.geometry.Nxi-2 ; % for removing boundary points after shifting
        options = optimoptions('fsolve', 'Display', 'off') ;
        Ddiags = ones(NxiExtended,1)*[-1,1]/(2*params.geometry.dxi) ;
        D = spdiags(Ddiags, [-1,1], NxiExtended, NxiExtended) ;
        DU = D*U ;
        shiftMatrix = shiftMatrixGenerator(params, -urTempOld(end), NxiExtended, neededPointsOnTheLeft) ;
        shifts = NaN(nTimeSteps, 1) ;
        amplitudes = NaN(r, nTimeSteps) ;
        shifts(1) = urTempOld(end) ;
        amplitudes(:,1) = urTempOld(1:end-1) ;
        for i=2:nTimeSteps
            dt = t(i)-t(i-1) ;
            shiftedModesOld = shiftMatrix{1}*U ;
            shiftedDerivativeModesOld = shiftMatrix{1}*DU ;
            M11old = shiftedModesOld'*shiftedModesOld ;
            M12old = -shiftedModesOld'*shiftedDerivativeModesOld*urTempOld(1:end-1) ;
            M22old = (shiftedDerivativeModesOld*urTempOld(1:end-1))'*(shiftedDerivativeModesOld*urTempOld(1:end-1)) ;
            MOld = [M11old,M12old; M12old',M22old] ;
            A1old = shiftedModesOld'*A*shiftedModesOld ;
            A2old = -urTempOld(1:end-1)'*shiftedDerivativeModesOld'*A*shiftedModesOld ;
            AOld = [A1old,zeros(r,1); A2old,0] ;
            B1old = shiftedModesOld'*B ;
            B2old = -urTempOld(1:end-1)'*shiftedDerivativeModesOld'*B ;
            BOld = [B1old; B2old] ;
            residual = @(urNew)helperForSPODROMtimeIntegrationViaImplTrapezoidalRule(urNew, urTempOld, dt, shiftMatrixGenerator, params, NxiExtended, neededPointsOnTheLeft, U, DU, A, B, g(t(i-1)), g(t(i)), MOld, AOld, BOld) ;
            urTempNew = fsolve(residual, urTempOld, options) ;
            shifts(i) = urTempNew(end) ;
            amplitudes(:,i) = urTempNew(1:end-1) ;
            shiftMatrix = shiftMatrixGenerator(params, -urTempNew(end), NxiExtended, neededPointsOnTheLeft) ;
            if(~mod(i-1,writeFrequency))
                currentIndex = (i-1)/writeFrequency ;
                u(:,1+currentIndex) = (shiftMatrix{1}*U)*urTempNew(1:end-1) ;
            end
            urTempOld = urTempNew ;
        end
    otherwise
        error('Unknown projection scheme!')
end