function minRelError = create_example_func(simulateADE, identifyModes, inputStruct, fileName)

restoredefaultpath

save_ADEsolution = false ;
save_modes = false;

if(nargin>3)
    if(simulateADE)
        params.write.fileName.ADE = fileName ;
        save_ADEsolution = true ;
    else
        params.write.fileName.sPOD = fileName ;
        save_modes = true;
    end
end

path(path, genpath('./sPOD/LIB/HELPER'))

%% ADE settings

params.time.dt = 0.001*round(1000/(inputStruct.Nxi-1)) ;
params.data.velocity         = 1 ;
params.data.diffusionParam = 0.001 ; % 0.02
params.data.IC.amplitude = 0.5 ;
params.data.IC.relativeStartingPoint = 0.5 ; % 0.5
params.data.IC.relativeStandardDerivation = 0.02 ;
params.data.BC.amplitude = 0.5 ; % 0.5
params.data.BC.startingPoint = 0.2 ;
params.data.BC.relativeStandardDerivation = 0.02 ;
params.geometry.Nxi = inputStruct.Nxi       ;
params.geometry.L   = 1       ; 
params.time.endTime = 1.5 ; % 1.2
params.plot.showPlots = giveStructValue(inputStruct, 'plot', 'showPlots', 'off') ;
params.write.frequency = 1 ; % max(1,round(1/params.simulation.cfl))
params.projection.mode = giveStructValue(inputStruct, 'ADE', 'projectionMode', 'off') ;

if(~strcmp(params.projection.mode,'off'))
    params.projection.modeFile = inputStruct.modeFile ;
    params.projection.computeError = true ;
    params.projection.referenceSolution = inputStruct.referenceSolution ;
end

%% sPOD settings

params.mode.method = giveStructValue(inputStruct, 'modeDecomp', 'method', 'sPOD') ;
params.data.name      = inputStruct.dataName ;
params.mode.nModes               = inputStruct.nModes     ;
params.mode.prtlevel              = 0 ; % if you want to see the iterations of the optimization, choose value 1
params.mode.maxIter = 20000 ;
params.mode.punishCoeff = 1e-7 ;
params.SLdetection.velocity  = 1       ;
params.mode.startValue = giveStructValue(inputStruct, 'sPOD', 'startValue', 'naiveDecomposition') ;
params.mode.loadPsiFrom = inputStruct.loadPsiFrom ;
params.mode.costFunctional =  @costFunctionalForOptimizedSPOD_extended ; % @costFunctionalForOptimizedSPOD_extended, costFunctionalForOptimizedSPOD_extended_punishDphi

%-------------------------------------------------------------------------------
%% Simulate FOM or ROM
%-------------------------------------------------------------------------------

if(simulateADE)
    status1 = mkdir('advectionDiffusionEq','DATA') ;
    status2 = mkdir('sPOD','DATA') ;
    status3 = mkdir('sPOD/DATA','advectionDiffusion') ;
    path(path, genpath('./advectionDiffusionEq/LIB/'))
    path(path, genpath('./advectionDiffusionEq/DATA/'))
    
    minRelError = NaN ;
    advectionDiffusionEq_main

    if(save_ADEsolution)
        saveData
    end
end

%-------------------------------------------------------------------------------
%% Shifted POD (or alternatively POD) offline
%-------------------------------------------------------------------------------

if(identifyModes)
    restoredefaultpath
    path(path, genpath('./sPOD/LIB/'))
    status2 = mkdir('sPOD','DATA') ;
    status3 = mkdir('sPOD/DATA','advectionDiffusion') ;

    load_extern
    if(strcmp(params.mode.method,'sPOD'))
        sPOD_main
        if(save_modes)
            saveData
        end
    else
        POD_main
        if(save_modes)
            saveDataPOD
        end
    end
end
