function [alpha, b, grad] = coefficientsForOptimizedSPOD_extended(Psi, pars)

snapshotMatrix             = pars.snapshotMatrix                         ;
shiftMatrices              = pars.shiftMatrices                          ;

Nxi = pars.params.geometry.Nxi ;
nModes = pars.params.mode.nModes ;
nTimeSteps    = pars.params.time.nTimeSteps ;

alpha = NaN(nModes, nTimeSteps) ;
b     = NaN(nModes, nTimeSteps) ;
K_i   = NaN(Nxi, nModes)        ;
grad  = zeros(numel(Psi), 1)    ;

NxiExtended =pars.NxiExtended ;
PsiReshaped = packPsi(Psi, NxiExtended, pars.params) ;

for i=1:nTimeSteps
    for k=1:nModes
        K_i(:,k) = shiftMatrices{i}*PsiReshaped(:,k) ;
    end
    b(:,i)                 = K_i'*snapshotMatrix(:,i)                                            ;
    [Ui, Sigmai, V1i, V2i] = truncatedSVD(K_i'*K_i)                                              ;
    alpha(:,i)             = V1i*(Sigmai\(Ui'*K_i'*snapshotMatrix(:,i)))+V2i*ones(size(V2i,2),1) ;
    temp = 0 ;
    for j=1:nModes
        temp                 = temp+alpha(j,i)*K_i(:,j) ;
    end  
    for k=1:nModes
        currentIndices = NxiExtended*(k-1)+(1:NxiExtended) ;
        grad(currentIndices) = grad(currentIndices)+2*shiftMatrices{i}'*alpha(k,i)*(temp-snapshotMatrix(:,i)) ;
    end
end