function [sPODsystem, qsTilde, relError] = subspaceDecompositionBySPODoptimization_extended(params, qs, cs)  

params.mode.tol                  = 1e-5                            ;
params.mode.nvec                 = 4                               ;

errorMeasure = @relErrorADE ;
Nxi          = params.geometry.Nxi                                                  ;
nTimeSteps   = params.time.nTimeSteps                                               ;
nModes       = params.mode.nModes                                                   ;
costFunctional = giveStructValue(params, 'mode', 'costFunctional', @costFunctionalForOptimizedSPOD_extended) ;
startValue = giveStructValue(params, 'mode', 'startValue', 'naiveDecomposition') ;

qsTilde                    = zeros(Nxi, nTimeSteps)        ;
[sPODsystem, params] = initShiftMatrix(params, -cs)     ; % the minus sign is on purpose since the transpose is used in multiShiftReduce3Nxi  

NxiExtended = computeNxiExtended(cs, params) ;

switch startValue
    case 'naiveDecomposition'
        naiveSystem           = multiShiftReduceExtended(params, sPODsystem, qs, nModes) ;
        x0 = reshape(naiveSystem.U, [], 1) ;
        clear naiveSystem
    case 'loadCoarsePsi'
        S = load(params.mode.loadPsiFrom) ;
        fineModes = refineModes(S.sPodSystem.Psi, NxiExtended) ;
        x0 = reshape(fineModes, [], 1) ;
        clear S
    otherwise
        error('Method for determining the start value is unknown.')
end

options.x0      = x0               ;
options.nstart  = 1                ;
options.normtol = params.mode.tol  ;
options.nvec    = params.mode.nvec ;
options.maxit = params.mode.maxIter ;
options.prtlevel = giveStructValue(params, 'mode', 'prtlevel', 1) ;

pars.nvar           = length(options.x0)                                                             ;
pars.NxiExtended = NxiExtended ;
pars.snapshotMatrix = qs                                                                    ;
pars.shiftMatrices  = sPODsystem.shiftMatrices                                                       ;
pars.params         = params                                                                         ;
pars.fgname         = @(x,inputPars)costFunctional(x,inputPars) ;

x                        = bfgs(pars, options)               ;
sPODsystem.Psi = packPsi(x, NxiExtended, params) ;
[sPODsystem.alpha, ~, ~] = coefficientsForOptimizedSPOD_extended(x, pars) ;

for i=1:nTimeSteps
    for k=1:nModes
        qsTilde(:,i) = qsTilde(:,i)+sPODsystem.alpha(k,i)*sPODsystem.shiftMatrices{i}*sPODsystem.Psi(1:NxiExtended,k) ;
    end
end

relError = errorMeasure(qs, qsTilde-qs) ;